import java.awt.Point;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.commons.math.stat.regression.SimpleRegression;

import PER.rover.TurnHeadAction;
import PER.rover.control.RoverState;

/**
 * 
 */

/**
 * IRScanner is a helper class for actually scanning 
 * @author dlariviere
 *
 */
public class IRScanner {
	private static int HEAD_TILT = 0; //keep head level with ground
	private static int MAX_RANGE = 135; //based on values used in ScanAction for converting raw IR readings into centimeters
	private static int MIN_RANGE = 25; 
	
	/**
	 * Converts raw IR reading into CM reading
	 * Taken from ScanAction in PER api. Wanted double accuracy (instead of int, previously provided). 
	 * Accuracy/error of raw data unknown, so not sure if this is actually warranted or a completely useless increase in precision
	 * @param scan
	 * @return
	 */
	public static double translateScan(int scan) {
		return Math.round(2.9318e-6*Math.pow(scan, 4)-.00111014*Math.pow(scan, 3)+
	            .156177*Math.pow(scan, 2)-10.1177*scan+298.26);
	}
	
	/**
	 * 
	 * @param perCon PERController
	 * @param gui MoverGUI
	 * @param startingAngle starting angle to start scanning at
	 * @param endingAngle angle not to exceed 
	 * @param increment angle to continually increase starting angle by until current angle exceeds endingAngle
	 * @return an ArrayList of "valid" Point2D.Double points
	 */
	public static ArrayList scan(PERController perCon, MoverGUI gui, int startingAngle, int endingAngle, int increment) {
		RoverState rovState = perCon.rov.state; 
		TurnHeadAction turnHead; 
		int curRange;
		double curRangeCM, x, y; 
		double curAngleRads; //current angle in radians
		ArrayList validPoints = new ArrayList(); 
		Point2D.Double p; 
		
		for (int curAngle=startingAngle; curAngle<endingAngle; curAngle+= increment) {
			//pan the PER's head
			turnHead = new TurnHeadAction(curAngle, HEAD_TILT); 
			perCon.executeAction(turnHead); 
			
			//measure current distance
			curRange = rovState.getRange(); 
			curRangeCM = translateScan(curRange); 
			curAngleRads = Math.toRadians(curAngle); 
			
			//process
			if (curRange>MIN_RANGE && curRange<MAX_RANGE) { 
				System.out.println("IRScanner: " + curAngle + ", " + curRangeCM);
				x = (int) (Math.cos(curAngleRads) * (double) curRangeCM);
				y = (int) (Math.sin(curAngleRads) * (double) curRangeCM); 
				p = new Point2D.Double(x, y);
				validPoints.add(p); 
				gui.addObstaclePoint(p); 
			}
			else {
				System.out.println("IRScanner: Value out of range"); 
			}
		}
		Iterator it = validPoints.iterator();
		SimpleRegression regXY = new SimpleRegression(); 
		SimpleRegression regYX = new SimpleRegression(); 
		
		while (it.hasNext()) {
			p = (Point2D.Double) it.next(); 
			System.out.println(p.getX() + "," + p.getY());
			regXY.addData(p.getX(), p.getY()); 
			regYX.addData(p.getY(), p.getX()); 
		}
		
		System.out.println("y = " + regXY.getSlope() + "x +" + regXY.getIntercept() + ";\t" + "angle: " + Math.toDegrees(Math.atan(regXY.getSlope())) + "; error" + regXY.getMeanSquareError()); 
		System.out.println("y = " + regYX.getSlope() + "x +" + regYX.getIntercept() + ";\t" + "angle: " + Math.toDegrees(Math.atan(1.0/regYX.getSlope())) + "; error" + regYX.getMeanSquareError()); 
		return validPoints; 
	}
}
