//package PER.basicGUI;

import PER.rover.*;

import java.lang.Math; 
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.File;
import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JPanel;

import org.apache.commons.math.stat.regression.SimpleRegression;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

class PathLocation
{
	int px,py;
	PathLocation next;
	public PathLocation(int x, int y)
	{
		px=x;
		py=y;
		next=null;
	}
	public void add(PathLocation newPoint)
	{
		PathLocation iter=this;
		while (iter.next!=null)
			iter=iter.next;
		iter.next=newPoint;
	}
}

class DisplayPic extends JFrame { 
	public DisplayPic(BufferedImage img, String title) {
		super(title);
		JPanel panel = new JPanel(); 
		JLabel label = new JLabel(); 
		label.setIcon(new ImageIcon(img));
		panel.add(label);
		this.setContentPane(panel);
		this.pack();
		this.setSize(new Dimension(img.getWidth(),img.getHeight()));
		this.show();
	}
}

public class MoverGUI extends JFrame implements MouseListener, MouseMotionListener, ActionListener {
	public static boolean USE_ROBOT = true; 
	
	private PERController perController; 
	
	int mouseX,mouseY;
	int robotX,robotY;
	int winSizeX,winSizeY;
	double robotTheta=0;
	
	PathLocation myPath;
	private static int SCALE = 2; //scale factor
	private Panel mainPanel;
	
	private JButton resetPathButton;
	private JButton followPathButton;
	private JButton wallFollowButton; 
	private JCheckBox closePathCheckbox;
	private JCheckBox takeImagesCheckbox;
	
	private JLabel robotLocationLabel;
	private JLabel robotOrientationLabel; //used in info panel to display info about robot	
	
	private JTextField perIPTextField; 
	private JButton connectToPERButton; 
	
	private ArrayList obstaclePoints; //arraylist of Point2D's representing points scanned directly in front of the PER
	
	private static int INIT_WINDOW_WIDTH = 600; 
	private static int INIT_WINDOW_HEIGHT = 500; 
	
	public MoverGUI(PERController controller) {
		super("PER Waypoint Mover");
		perController = controller; 
		obstaclePoints = new ArrayList(); 
		
		mainPanel = new Panel();
		this.add(mainPanel);
		
		Panel bottomPanel = new Panel();
		bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.Y_AXIS));
		bottomPanel.add(getConnectPanel()); 
		bottomPanel.add(getButtonPanel());
		bottomPanel.add(getInfoPanel());
		this.getContentPane().add("South", bottomPanel);
		
		mainPanel.setBackground(Color.white);
		mainPanel.setForeground(Color.black);
		mainPanel.addMouseListener(this);
		mainPanel.addMouseMotionListener(this);
		
		mouseX=mouseY=0; 
		winSizeX=INIT_WINDOW_WIDTH;
		winSizeY=INIT_WINDOW_HEIGHT; 
		robotX=INIT_WINDOW_WIDTH/2;
		robotY=INIT_WINDOW_HEIGHT/3; 
		robotTheta = 0; 
		
		myPath = new PathLocation(robotX, robotY);
		
		this.pack();
		this.setSize(new Dimension(INIT_WINDOW_WIDTH,INIT_WINDOW_HEIGHT));
		this.show();
		
		mainPanel.repaint(); 
		this.getContentPane().repaint(); 
		//repaint(); 
	}
	
	/**
	 * Button Panel contains the "Follow Path" and "Reset" buttons,
	 * as well as the "Close Path" and "Take Images" checkboxes
	 * @return
	 */
	private JPanel getButtonPanel() { 
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		
		resetPathButton = new JButton("Reset");
		followPathButton = new JButton("Follow path");
		wallFollowButton = new JButton("IR Scan"); 
		closePathCheckbox = new JCheckBox("Close Path");
		takeImagesCheckbox = new JCheckBox("Take Images");
		
		buttonPanel.add(resetPathButton);
		buttonPanel.add(followPathButton);
		buttonPanel.add(wallFollowButton); 
		buttonPanel.add(closePathCheckbox);
		buttonPanel.add(takeImagesCheckbox);
		
		resetPathButton.addActionListener(this);
		followPathButton.addActionListener(this);
		wallFollowButton.addActionListener(this); 
		closePathCheckbox.addActionListener(this);
		takeImagesCheckbox.addActionListener(this);
		return buttonPanel; 
	}
	
	/**
	 * The Connect Panel is a small panel which displays a label ("Rover IP"),
	 * a textbox to hold the IP, and a button ("Connect"). 
	 * @return
	 */
	private JPanel getConnectPanel() { 
		JPanel connectionPanel = new JPanel(); 
		connectionPanel.add(new JLabel("Rover IP:"));
		
		perIPTextField = new JTextField(perController.rov.getDefaultIP(), 13);
		connectionPanel.add(perIPTextField); 
		
		connectToPERButton = new JButton("Connect to PER"); 
		connectToPERButton.addActionListener(this); 
		connectionPanel.add(connectToPERButton); 
		
		return connectionPanel; 
	}
	
	/**
	 * Initializes an info panel to be displayed at the bottom of the screen
	 * displaying the value sof robotX, robotY, and robotTheta
	 */
	private JPanel getInfoPanel() { 
		JPanel infoPanel = new JPanel(new FlowLayout());
		robotLocationLabel = new JLabel();
		robotOrientationLabel = new JLabel();
		updateInfoPanel(); 
		infoPanel.add(robotLocationLabel);
		infoPanel.add(robotOrientationLabel);
		infoPanel.show();
		
		return infoPanel; 
	}
	
	/**
	 * called to update the textboxes showing the robot's current location/orientation
	 */
	private void updateInfoPanel() { 
		robotLocationLabel.setText("Robot Location: " + robotX + "," + robotY);
		robotOrientationLabel.setText("Robot Orientation: " + (double)robotTheta + "\u00B0");
	}
	
	public void drawRobot(Graphics2D g2) {
		g2 = (Graphics2D) mainPanel.getGraphics();
		AffineTransform saveXform = g2.getTransform();
		AffineTransform robotXform = new AffineTransform();
		robotXform.translate(robotX,robotY);
		robotXform.rotate(Math.toRadians(robotTheta));
		g2.transform(robotXform);
		
		g2.setPaint(Color.red);
		g2.fill (new Rectangle2D.Double(-10,-10,20,20));
		AffineTransform tipXform = new AffineTransform();
		tipXform.translate(10,0);
		tipXform.rotate(Math.toRadians(45));
		g2.transform(tipXform);
		g2.fill (new Rectangle2D.Double(-10,-10,20,20));
		
		g2.setTransform(saveXform);
	}
	
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) mainPanel.getGraphics();
		
		g2.setBackground(Color.white);
		g2.setPaint(Color.white);
		g2.fillRect(0, 0, mainPanel.getWidth(), mainPanel.getHeight());
		
		g2.setPaint(Color.black);
		winSizeY = mainPanel.getHeight();
		winSizeX = mainPanel.getWidth(); 
		g2.draw(new Line2D.Double(mouseX,0,mouseX,winSizeY));	
		g2.draw(new Line2D.Double(0,mouseY,winSizeX,mouseY));
		
		g2.setPaint(Color.blue);		
		PathLocation iter=myPath;
		while(iter!=null)
		{
			g2.fill (new Rectangle2D.Double(iter.px-5,iter.py-5,10,10));
			if (iter.next!=null) {
				g2.draw(new Line2D.Double( iter.px , iter.py , iter.next.px , iter.next.py));
			}
			iter=iter.next;		
		}
		
		//draw obstacle points in green
		g2.setPaint(Color.green);
		Iterator obstacleIt = obstaclePoints.iterator();
		int OBSTACLE_RADIUS = 4; 
		while (obstacleIt.hasNext()) { 
			Point2D obPoint = (Point2D) obstacleIt.next(); 
			g2.fillOval((int)obPoint.getX()-(OBSTACLE_RADIUS/2), (int)obPoint.getY()-(OBSTACLE_RADIUS/2), OBSTACLE_RADIUS, OBSTACLE_RADIUS); 
		}
		
		drawRobot(g2);
	}
	
	public void mouseClicked(MouseEvent e) 
	{
		int newX=e.getX();
		int newY=e.getY();
		
		if (myPath==null)
			myPath=new PathLocation(newX,newY);
		else
			myPath.add( new PathLocation(newX,newY) );
		mainPanel.repaint(); 
		repaint(); 
	}
	
	public void mouseMoved(MouseEvent e)
	{
		mouseX=e.getX();
		mouseY=e.getY();
		mainPanel.repaint(); 
		repaint(); 
	}
	
	public void followPath() { 		
		int dX, dY;
		double theta, initTheta = 0;
		int waypoint = 1; 
		PathLocation startingPath, endLoop; //used for storing starting point to do closed path
		startingPath = null; 
		
		if (isClosingPath()) { //will become if closed path
			//need to remember starting point so that we can go back to it once we are done
			initTheta = robotTheta; 
			startingPath = myPath;
			myPath.add(new PathLocation(robotX, robotY));
		}
		
		myPath = myPath.next; //the first waypoint is always the current point
		while (myPath!=null) { 	
			dX = myPath.px - this.robotX;
			dY = myPath.py - this.robotY; 
			dX /= SCALE;
			dY /= SCALE; 
			
			theta = Math.atan2(dY, dX); //calculate angle to move to dx,dy relative to absolute origin
			theta -= Math.toRadians(robotTheta); //take into account robot's current orientation
			robotTheta += (double) (Math.toDegrees(theta)); //increment robot's orientation to what will be the new value after turning
			theta *= -1*(180/3.14159); 
			
			TurnToAction turn = new TurnToAction((int)theta); //generate turn action to rotate robot to point in direction of desired motion	
			int moveDistance = (int)Math.sqrt(Math.pow(dY,2)+Math.pow(dX,2)); 
			
			if (USE_ROBOT) {
				perController.executeAction(turn); //turn towards desired direction of motion
			}
			DriveToAction move = new DriveToAction(moveDistance, 0,	DriveToAction.STATIC_SAFETY, isTakePictures());
			
			
			//if executeAction returns false, then the robot stopped before completing the move because of obstacle
			if (USE_ROBOT && !perController.executeAction(move)) {
				//get the distance the robot did indeed move and update current known position with that
				moveDistance = perController.rov.highLevelState.getDist();
				this.robotX += SCALE * moveDistance * Math.cos(Math.toRadians(robotTheta));
				this.robotY += SCALE * moveDistance * Math.sin(Math.toRadians(robotTheta));
				System.out.println("PER Stopped Early"); 
			}
			
			//else entire distance was moved, properly update location
			else {
				this.robotX += SCALE*dX; 
				this.robotY += SCALE*dY; 
			}
			
			//if we are taking pictures at waypoints, then snap one!
			if (USE_ROBOT && isTakePictures() && waypoint>0) {
				displayPic(waypoint); 
			}
			
			myPath=myPath.next; 
			waypoint++; 
		}
		
		myPath = new PathLocation(robotX, robotY);
		if (isClosingPath()) { 
			myPath = startingPath; 
		}
		repaint(); 
	}
	
	
	public void displayPic(final int wp) { //pass it waypoint # (will be displayed in Pic title)
		 Thread currThread = new Thread() {
			 public void run(){
				 int pointNum = wp; 
				 BufferedImage image = perController.rov.takePicture(perController.rov.state.getPan(), perController.rov.state.getTilt(), 640, 480);
				 if(image == null) {
					 System.out.println("Error taking picture: "+ActionConstants.getErrorText(perController.rov.state.getStatus()));
				 }
				 else{
					 System.out.println("Picture complete");
					 new DisplayPic(image, "Waypoint " + pointNum);
				 }
			 }
		 };
		 
		 //run the command
		 try{
			 currThread.start();
			 System.out.println("Taking picture at waypoint " + wp);
		 }
		 catch (java.lang.IllegalThreadStateException e) {
			 System.out.println("Error taking picture @ waypoint " + wp);
		 }
	}
	
	/**
	 * Used to query value of Closed Path Checkbox
	 * @return true if the closePathCheckbox is checked, false otherwise
	 */
	public boolean isClosingPath() {
		return closePathCheckbox.isSelected(); 
	}
	
	/**
	 * Used to query value of Take Pictures Checkbox
	 * @return true if the takeImagesCheckbox is checked, false otherwise
	 */
	public boolean isTakePictures() {
		return takeImagesCheckbox.isSelected();
	}
	
	public void resetPath() { 
		myPath = new PathLocation(robotX, robotY);
		repaint(); 
	}
	
	/**
	 * Debugging code - used to generate fake scans when not actually connected to the PER
	 * @return
	 */
	private ArrayList fakeIRScan() { 
		ArrayList scanpoints = new ArrayList(); 
		for (double i=-30; i<30; i+=5.0) { 
			scanpoints.add(new Point2D.Double(60, i));
		}
		Point2D.Double p; 
		Iterator it = scanpoints.iterator();
		SimpleRegression regXY = new SimpleRegression(); 
		SimpleRegression regYX = new SimpleRegression(); 
		
		while (it.hasNext()) {
			p = (Point2D.Double) it.next(); 
			System.out.println(p.getX() + "," + p.getY());
			regXY.addData(p.getX(), p.getY()); 
			regYX.addData(p.getY(), p.getX()); 
		}
		
		System.out.println("y = " + regXY.getSlope() + "x +" + regXY.getIntercept() + ";\t" + "angle: " + Math.toDegrees(Math.atan(regXY.getSlope())) + "; error" + regXY.getMeanSquareError()); 
		System.out.println("y = " + regYX.getSlope() + "x +" + regYX.getIntercept() + ";\t" + "angle: " + Math.toDegrees(Math.atan(1.0/regYX.getSlope())) + "; error" + regYX.getMeanSquareError()); 
		
		return scanpoints; 
	}
	
	public void addObstaclePoint(Point2D p) {
		double relX, relY; 
		double cos, sin; 
		int globalX, globalY; 
		Point2D.Double globalPoint; 
		
		if (!obstaclePoints.contains(p)) {
			relX = p.getX() * SCALE;
			relY = p.getY() * SCALE; 
			cos = Math.cos(Math.toRadians(-1.0 * robotTheta));
			sin = Math.sin(Math.toRadians(-1.0 * robotTheta)); 
			globalX = (int) (relX * cos - relY * sin); 
			globalY = (int) (relX * sin + relY * cos); 
			globalPoint = new Point2D.Double(globalX + robotX, robotY - globalY); 
			System.out.println(p + " --> " + globalPoint); 
			
			this.obstaclePoints.add(globalPoint); 
			repaint(); 
		}
	}
	
	public void scanObstacle() { 
		System.out.println("Scanning obstacle");
		boolean trueScan = true; 

		ArrayList perRelativePoints;
		if (trueScan) {
			perRelativePoints = IRScanner.scan(perController, this, -20, 20, 1); // fakeIRScan(); //
		}
		else { 
			perRelativePoints = fakeIRScan(); 
			Iterator it = perRelativePoints.iterator();
			while (it.hasNext()) { 
				Point2D p = (Point2D) it.next(); 
				addObstaclePoint(p); 
			}
		}
	}
	
	/**
	 * Called when GUI objects which registered MoverGUI as their actionListener
	 * receive an an event
	 */
	public void actionPerformed(final ActionEvent event)
	{
		Thread newThread = new Thread() {
			 public void run(){
				 seperateThreadedActionPerformed(event); 
			 }
		 };
		 
		 //run the command
		 try {
			 newThread.start();
		 }
		 catch (java.lang.IllegalThreadStateException e) {
			 System.out.println("Error handling perfomed action" + e.getMessage());
		 }
	}
	
	/**
	 * seperateThreadedActionPerformed does the actual work of actionPerfored, but is intended
	 * to be called within a seperate thread. Doing so allows the GUI thread to continue working
	 * while the additional processing which occurs as a result of a particular action (pushing a 
	 * button, etc) can perform simultaneously. Thus the GUI can be updated while we are 
	 * updating it, rather than having to wait for the below functions to finish before
	 * the GUI can actually be updated. 
	 * @param e
	 */
	public void seperateThreadedActionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("Reset")) //if reset button was clicked
		{
			resetPath(); 
		}
		if (e.getActionCommand().equals("Follow path")) //if follow path button was clicked
		{
			followPath(); 		
		}
		if (e.getActionCommand().equals("Connect to PER")) //if follow path button was clicked
		{
			perController.connectToPER(perIPTextField.getText()); 		
		}
		if (e.getActionCommand().equals("IR Scan")) { 
			scanObstacle(); 
		}
	}
	
	public void mousePressed(MouseEvent e) 
	{
	}
	
	public void mouseReleased(MouseEvent e) 
	{
	}
	
	public void mouseEntered(MouseEvent e) 
	{
	}
	
	public void mouseExited(MouseEvent e) 
	{
	}
	
	public void mouseDragged(MouseEvent e)
	{
	}
}
